#  Parts of this wrapper are inspired by atlas, an easy-to-use metagenomic pipeline based on snakemake.
#  Go check it out on https://github.com/metagenome-atlas/atlas
import logging
import os
import sys
import click
import subprocess
import datetime


logging.basicConfig(
    level=logging.INFO,
    datefmt="%Y-%m-%d %H:%M",
    format="[%(asctime)s %(levelname)s] %(message)s",
)

"""
Functions
"""


def get_version():
    thisdir = os.path.abspath(os.path.dirname(__file__))
    init = os.path.join(thisdir, "__init__.py")
    return open(init).readline().split(" = ")[1].replace('"', "")


def get_snakefile():
    thisdir = os.path.abspath(os.path.dirname(__file__))
    sf = os.path.join(thisdir, "Snakefile")
    if not os.path.exists(sf):
        sys.exit(f"Unable to locate the Snakemake workflow file at {sf}")
    return sf


"""
Main
"""

version = get_version()

desc = f"""
       ░█▀█░█▀▀░█▀▀░█▀▀░█▄█░█▀▄░█░░░█░█░░░█▀▀░▀█▀░█▀█░█▀▄░█▀▀░█▀▄
       ░█▀█░▀▀█░▀▀█░█▀▀░█░█░█▀▄░█░░░░█░░░░█▀▀░░█░░█░█░█░█░█▀▀░█▀▄
       ░▀░▀░▀▀▀░▀▀▀░▀▀▀░▀░▀░▀▀░░▀▀▀░░▀░░░░▀░░░▀▀▀░▀░▀░▀▀░░▀▀▀░▀░▀
       \nv{version}

      Snakemake pipeline to download genome assemblies from NCBI

      github: https://github.com/metagenlab/assembly_finder
      """
CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])


@click.command(help=desc, context_settings=CONTEXT_SETTINGS)
@click.option(
    "-i",
    "--input",
    type=str,
    help="path to assembly_finder input table or list of entries",
    required=True,
)
@click.option(
    "-e",
    "--extensions",
    type=str,
    help="suffix of files to download from NCBI's ftp",
    default="assembly_report.txt,genomic.fna.gz",
    show_default=True,
)
@click.option("-o", "--output", help="Output directory", type=str)
@click.option(
    "-n",
    "--dryrun_status",
    is_flag=True,
    default=False,
    show_default=True,
    help="Snakemake dryrun to see the scheduling plan",
)
@click.option(
    "-t",
    "--threads",
    type=int,
    help="number of threads to allow for the workflow",
    default=2,
    show_default=True,
)
@click.option("-nk", "--ncbi_key", type=str, help="ncbi key for Entrez", default="none")
@click.option(
    "-ne", "--ncbi_email", type=str, help="ncbi email for Entrez", default="none"
)
@click.option(
    "-db",
    "--database",
    type=str,
    help="download from refseq or genbank",
    default="refseq",
    show_default=True,
)
@click.option(
    "-id",
    "--uid",
    help="are inputs UIDs",
    type=str,
    default=False,
    show_default=True,
)
@click.option(
    "-rc",
    "--refseq_category",
    type=str,
    help="select reference, representative or all",
    default="all",
    show_default=True,
)
@click.option(
    "-al",
    "--assembly_level",
    type=str,
    help="select complete, chromosome, scaffold, contig or all",
    default="complete",
    show_default=True,
)
@click.option(
    "-an",
    "--annotation",
    type=str,
    help="select assemblies with annotation",
    default=False,
    show_default=True,
)
@click.option(
    "-ex",
    "--exclude",
    type=str,
    help="filter to exclude assemblies (example: exclude from metagenomes)",
    default="metagenome",
    show_default=True,
)
@click.option(
    "-r",
    "--filter_rank",
    help="rank to filter by (example: species)",
    default="none",
    type=str,
    show_default=True,
)
@click.option(
    "-nr",
    "--n_by_rank",
    help="Max number of genome by target rank (example: 1 per species)",
    type=str,
    default="none",
    show_default=True,
)
@click.option(
    "-nb",
    "--n_by_entry",
    help="Number of genomes per entry",
    type=str,
    default="all",
    show_default=True,
)
@click.option(
    "-et",
    "--ete_db",
    type=str,
    default=os.path.join(os.environ["HOME"], ".etetoolkit"),
    help="path where to save/find ete taxa.sqlite file",
)
@click.argument("snakemake_args", nargs=-1, type=click.UNPROCESSED)
@click.version_option(version, "-v", "--version")
def cli(
    input,
    output,
    extensions,
    dryrun_status,
    threads,
    ncbi_key,
    ncbi_email,
    database,
    uid,
    refseq_category,
    assembly_level,
    annotation,
    exclude,
    filter_rank,
    n_by_rank,
    n_by_entry,
    ete_db,
    snakemake_args,
):
    if not output:
        output = datetime.datetime.today().strftime("%Y-%m-%d")

    if dryrun_status:
        dryrun = "-n"
    else:
        dryrun = ""

    if snakemake_args:
        args = " ".join(["--" + arg for arg in snakemake_args])
    else:
        args = ""

    cmd = (
        f"snakemake --snakefile {get_snakefile()} "
        f" --cores {threads} "
        f"all_download {dryrun} "
        f"--config ete_db={ete_db} "
        f"ncbi_key={ncbi_key} "
        f"ncbi_email={ncbi_email} "
        f"input={input} "
        f"exts={extensions} "
        f"nb={n_by_entry} "
        f"db={database} "
        f"uid={uid} "
        f"alvl={assembly_level} "
        f"excl={exclude} "
        f"rcat={refseq_category} "
        f"annot={annotation} "
        f"rank={filter_rank} "
        f"nrank={n_by_rank} "
        f"outdir={output} "
        f"{args}"
    )
    logging.info(f"Executing: {cmd}")
    try:
        subprocess.check_call(cmd, shell=True)
    except subprocess.CalledProcessError as e:
        # removes the traceback
        logging.critical(e)
        exit(1)


if __name__ == "__main__":
    cli()
